@@ -204,11 +204,11 @@ impl Backend {
 
         for (input_key, input) in input_registry.iter_inputs() {
             for (ident, audit) in self.audit_registry.iter_audits() {
-                registry.extend(audit.audit(
-                    ident,
-                    input,
-                    input_registry.get_config(input_key.group()),
-                )?);
+                registry.extend(
+                    audit
+                        .audit(ident, input, input_registry.get_config(input_key.group()))
+                        .await?,
+                );
             }
         }
 
@@ -275,7 +275,6 @@ impl From<Point> for lsp_types::Position {
     }
 }
 
-#[tokio::main]
 pub(crate) async fn run() -> Result<(), Error> {
     let stdin = tokio::io::stdin();
     let stdout = tokio::io::stdout();
