@@ -7,7 +7,7 @@ use std::{
 };
 
 use annotate_snippets::{Group, Level, Renderer};
-use anstream::{eprintln, println, stream::IsTerminal};
+use anstream::{eprintln, println, stderr, stream::IsTerminal};
 use anyhow::anyhow;
 use camino::Utf8PathBuf;
 use clap::{Args, CommandFactory, Parser, ValueEnum, builder::NonEmptyStringValueParser};
@@ -15,6 +15,7 @@ use clap_complete::Generator;
 use clap_verbosity_flag::InfoLevel;
 use etcetera::AppStrategy as _;
 use finding::{Confidence, Persona, Severity};
+use futures::stream::{FuturesOrdered, StreamExt};
 use github::{GitHubHost, GitHubToken};
 use indicatif::ProgressStyle;
 use owo_colors::OwoColorize;
@@ -493,18 +494,6 @@ pub(crate) enum FixMode {
     All,
 }
 
-pub(crate) fn tips(err: impl AsRef<str>, tips: &[impl AsRef<str>]) -> String {
-    // NOTE: We use secondary_title here because primary_title doesn't
-    // allow ANSI colors, and some of our errors contain colorized text.
-    let report = vec![
-        Group::with_title(Level::ERROR.secondary_title(err.as_ref()))
-            .elements(tips.iter().map(|tip| Level::HELP.message(tip.as_ref()))),
-    ];
-
-    let renderer = Renderer::styled();
-    renderer.render(&report).to_string()
-}
-
 /// State used when collecting input groups.
 pub(crate) struct CollectionOptions {
     pub(crate) mode_set: CollectionModeSet,
@@ -515,15 +504,16 @@ pub(crate) struct CollectionOptions {
 }
 
 #[instrument(skip_all)]
-fn collect_inputs(
+async fn collect_inputs(
     inputs: &[String],
     options: &CollectionOptions,
     gh_client: Option<&Client>,
 ) -> Result<InputRegistry, CollectionError> {
     let mut registry = InputRegistry::new();
 
+    // TODO: use tokio's JoinSet?
     for input in inputs.iter() {
-        registry.register_group(input, options, gh_client)?;
+        registry.register_group(input, options, gh_client).await?;
     }
 
     if registry.len() == 0 {
@@ -563,8 +553,8 @@ enum Error {
     /// An error while running an audit.
     #[error("{ident} failed on {input}")]
     Audit {
-        source: anyhow::Error,
         ident: &'static str,
+        source: anyhow::Error,
         input: String,
     },
     /// An error while rendering output.
@@ -575,10 +565,10 @@ enum Error {
     Fix(#[source] anyhow::Error),
 }
 
-fn run(app: &mut App) -> Result<ExitCode, Error> {
+async fn run(app: &mut App) -> Result<ExitCode, Error> {
     #[cfg(feature = "lsp")]
     if app.lsp.lsp {
-        lsp::run()?;
+        lsp::run().await?;
         return Ok(ExitCode::SUCCESS);
     }
 
@@ -622,7 +612,13 @@ fn run(app: &mut App) -> Result<ExitCode, Error> {
     // compose perfectly: `anstream` wants to strip all ANSI escapes,
     // while `tracing_indicatif` needs line control to render progress bars.
     // TODO: In the future, perhaps we could make these work together.
-    if matches!(color_mode, ColorMode::Never) {
+    //
+    // Also, we disable progress bars if stderr is not a terminal.
+    // Technically indicatif does this for us, but tracing_indicatif
+    // surfaces a bug when multiple spans are active and the
+    // output is not a terminal.
+    // See: https://github.com/emersonford/tracing-indicatif/issues/24
+    if matches!(color_mode, ColorMode::Never) || !stderr().is_terminal() {
         app.no_progress = true;
     }
 
@@ -715,7 +711,8 @@ fn run(app: &mut App) -> Result<ExitCode, Error> {
         app.inputs.as_slice(),
         &collection_options,
         gh_client.as_ref(),
-    )?;
+    )
+    .await?;
 
     let state = AuditState::new(app.no_online_audits, gh_client);
 
@@ -727,7 +724,8 @@ fn run(app: &mut App) -> Result<ExitCode, Error> {
         let span = info_span!("audit");
         span.pb_set_length((registry.len() * audit_registry.len()) as u64);
         span.pb_set_style(
-            &ProgressStyle::with_template("[{elapsed_precise}] {bar:!30.cyan/blue} {msg}").unwrap(),
+            &ProgressStyle::with_template("[{elapsed_precise}] {bar:!30.cyan/blue} {msg}")
+                .expect("couldn't set progress bar style"),
         );
 
         let _guard = span.enter();
@@ -742,22 +740,26 @@ fn run(app: &mut App) -> Result<ExitCode, Error> {
                 warn_once!("for more information, see: https://docs.zizmor.sh/usage/#yaml-anchors");
             }
 
+            let mut completion_stream = FuturesOrdered::new();
             let config = registry.get_config(input_key.group());
             for (ident, audit) in audit_registry.iter_audits() {
-                tracing::debug!("running {ident} on {input}", input = input.key());
+                tracing::debug!("scheduling {ident} on {input}", input = input.key());
 
-                results.extend(
-                    audit
-                        .audit(ident, input, config)
-                        .map_err(|err| Error::Audit {
-                            source: err,
-                            ident,
-                            input: input.key().to_string(),
-                        })?,
-                );
+                completion_stream.push_back(audit.audit(ident, input, config));
+            }
+
+            while let Some(findings) = completion_stream.next().await {
+                let findings = findings.map_err(|err| Error::Audit {
+                    ident: err.ident(),
+                    source: err.into(),
+                    input: input.key().to_string(),
+                })?;
+
+                results.extend(findings);
 
                 Span::current().pb_inc(1);
             }
+
             tracing::info!(
                 "🌈 completed {input}",
                 input = input.key().presentation_path()
@@ -802,14 +804,33 @@ fn run(app: &mut App) -> Result<ExitCode, Error> {
     }
 }
 
-fn main() -> ExitCode {
-    human_panic::setup_panic!();
+#[tokio::main]
+async fn main() -> ExitCode {
+    // NOTE: We only use human-panic on non-CI environments.
+    // This is because human-panic's output gets sent to a temporary file,
+    // which is then typically inaccessible from an already failed
+    // CI job. In those cases, it's better to dump directly to stderr,
+    // since that'll typically be captured by console logging.
+    if std::env::var_os("CI").is_some() {
+        std::panic::set_hook(Box::new(|info| {
+            let trace = std::backtrace::Backtrace::force_capture();
+            eprintln!("FATAL: zizmor crashed. This is a bug that should be reported.");
+            eprintln!(
+                "Please report to: {repo}",
+                repo = env!("CARGO_PKG_REPOSITORY")
+            );
+            eprintln!("Panic information:\n{}", info);
+            eprintln!("Backtrace:\n{}", trace);
+        }));
+    } else {
+        human_panic::setup_panic!();
+    }
 
     let mut app = App::parse();
 
     // This is a little silly, but returning an ExitCode like this ensures
     // we always exit cleanly, rather than performing a hard process exit.
-    match run(&mut app) {
+    match run(&mut app).await {
         Ok(exit) => exit,
         Err(err) => {
             eprintln!(
@@ -898,6 +919,24 @@ fn main() -> ExitCode {
 
                         Some(report)
                     }
+                    CollectionError::RemoteWithoutWorkflows(_, slug) => {
+                        let group = Group::with_title(Level::ERROR.primary_title(err.to_string()))
+                            .elements([
+                                Level::HELP.message(
+                                    format!(
+                                        "ensure that {slug} contains one or more workflows under `.github/workflows/`"
+                                    )
+                                ),
+                                Level::HELP.message(
+                                    format!("ensure that {slug} exists and you have access to it")
+                                )
+                            ]);
+
+                        let renderer = Renderer::styled();
+                        let report = renderer.render(&[group]);
+
+                        Some(report)
+                    }
                     _ => None,
                 },
                 _ => None,
