@@ -10,7 +10,6 @@ use crate::{
     finding::{Confidence, Finding, Persona, Severity},
     registry::input::{InputKey, InputRegistry},
     state::AuditState,
-    tips,
 };
 
 pub(crate) mod input;
@@ -41,12 +40,6 @@ impl AuditRegistry {
                     Err(AuditLoadError::Skip(e)) => {
                         tracing::debug!("skipping {audit}: {e}", audit = base::ident())
                     }
-                    Err(AuditLoadError::Fail(e)) => {
-                        return Err(anyhow::anyhow!(tips(
-                            format!("failed to load audit: {audit}", audit = base::ident()),
-                            &[format!("{e:#}"), format!("see: {url}", url = base::url())]
-                        )));
-                    }
                 }
             }};
         }
