@@ -3,7 +3,7 @@
 //! TODO: This file is too big; break it into multiple
 //! modules, one per audit/conceptual group.
 
-use crate::common::{input_under_test, zizmor};
+use crate::common::{OutputMode, input_under_test, zizmor};
 use anyhow::Result;
 
 #[test]
@@ -707,6 +707,21 @@ fn github_env() -> Result<()> {
             .run()?
     );
 
+    // Ensures that we produce a reasonable warning if the user gives us a
+    // `shell:` clause containing an expression.
+    insta::assert_snapshot!(
+        zizmor()
+            .output(OutputMode::Both)
+            .setenv("RUST_LOG", "warn")
+            .input(input_under_test("github-env/issue-1333/action.yml"))
+            .run()?,
+        @r"
+    🌈 zizmor v@@VERSION@@
+     WARN zizmor::audit::github_env: github-env: couldn't determine shell type for @@INPUT@@ step 0; assuming bash
+    No findings to report. Good job!
+    "
+    );
+
     Ok(())
 }
 
@@ -863,8 +878,25 @@ fn ref_version_mismatch() -> Result<()> {
     insta::assert_snapshot!(
         zizmor()
             .offline(false)
+            .output(crate::common::OutputMode::Both)
             .input(input_under_test("ref-version-mismatch.yml"))
-            .run()?
+            .run()?,
+        @r"
+    🌈 zizmor v@@VERSION@@
+     INFO audit: zizmor: 🌈 completed @@INPUT@@
+    warning[ref-version-mismatch]: detects commit SHAs that don't match their version comment tags
+      --> @@INPUT@@:22:77
+       |
+    22 |       - uses: actions/setup-node@1a4442cacd436585916779262731d5b162bc6ec7 # v3.8.1
+       |         -----------------------------------------------------------------   ^^^^^^ points to commit 5e21ff4d9bc1
+       |         |
+       |         is pointed to by tag v3.8.2
+       |
+       = note: audit confidence → High
+       = note: this finding has an auto-fix
+
+    2 findings (1 suppressed, 1 fixable): 0 informational, 0 low, 1 medium, 0 high
+    "
     );
 
     // Tags that point to other tags are handled correctly.
@@ -1078,6 +1110,7 @@ fn concurrency_limits() -> Result<()> {
     "
     );
 
+    // Note: per #1302, we intentionally don't produce findings here.
     insta::assert_snapshot!(
         zizmor()
             .input(input_under_test(
@@ -1085,19 +1118,7 @@ fn concurrency_limits() -> Result<()> {
             ))
             .args(["--persona=pedantic"])
             .run()?,
-        @r"
-    help[concurrency-limits]: insufficient job-level concurrency limits
-     --> @@INPUT@@:5:1
-      |
-    5 | / concurrency:
-    6 | |   group: ${{ github.workflow }}-${{ github.event.pull_request_number || github.ref }}
-    7 | |   cancel-in-progress: false
-      | |___________________________^ cancel-in-progress set to false
-      |
-      = note: audit confidence → High
-
-    1 finding: 0 informational, 1 low, 0 medium, 0 high
-    "
+        @"No findings to report. Good job!"
     );
 
     insta::assert_snapshot!(
@@ -1112,7 +1133,7 @@ fn concurrency_limits() -> Result<()> {
      --> @@INPUT@@:5:1
       |
     5 | concurrency: group
-      | ^^^^^^^^^^^^^^^^^^ concurrency is missing cancel-in-progress
+      | ^^^^^^^^^^^^^^^^^^ workflow concurrency is missing cancel-in-progress
       |
       = note: audit confidence → High
 
@@ -1120,5 +1141,38 @@ fn concurrency_limits() -> Result<()> {
     "
     );
 
+    insta::assert_snapshot!(
+        zizmor()
+            .input(input_under_test(
+                "concurrency-limits/jobs-missing-no-cancel.yml"
+            ))
+            .args(["--persona=pedantic"])
+            .run()?,
+        @r"
+  help[concurrency-limits]: insufficient job-level concurrency limits
+   --> @@INPUT@@:9:5
+    |
+  9 |     concurrency: group
+    |     ^^^^^^^^^^^^^^^^^^ job concurrency is missing cancel-in-progress
+    |
+    = note: audit confidence → High
+
+  help[concurrency-limits]: insufficient job-level concurrency limits
+    --> @@INPUT@@:1:1
+     |
+   1 | / name: Workflow with job 1 missing cancel-in-progress and job 2 missing concurrency
+   2 | | on: push
+   3 | | permissions: {}
+  ...  |
+  17 | |     - name: 2-ok
+  18 | |       run: echo ok
+     | |___________________^ missing concurrency setting
+     |
+     = note: audit confidence → High
+
+  2 findings: 0 informational, 2 low, 0 medium, 0 high
+    "
+    );
+
     Ok(())
 }
